using System;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace dict {

  /// <summary>Form for creating and selecting index files</summary>
  public class CfgForm : Form, IProgress {

    /// <summary>Input box for the index file</summary>
    private TextBox m_idxfile;

    /// <summary>Input box for the dictionary file</summary>
    private TextBox m_dictfile;

    /// <summary>Shows progress of the index creation</summary>
    private ProgressBar m_progress;

    /// <summary>Progress value to set in m_progress</summary>
    private int m_progressval;

    /// <summary>internal_prograss() as event</summary>
    /// <seealso cref="internal_progress">internal_progress</seealso>
    private EventHandler m_evprogress;

    /// <summary>Dictionary index creation engine</summary>
    private DictImport m_dictimport;

    /// <summary>The programm configuration</summary>
    private DictCfg m_cfg;

    /// <summary>Shows progress in the progress bar</summary>
    /// <remarks>
    /// Internal meaning: called from progress() using Invoke().
    /// Sets the progress bar value in m_progress from m_progressval.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void internal_progress(object sender, EventArgs e) {
      this.m_progress.Value = this.m_progressval;
    } // internal_progress()

    /// <summary>Shows progress in the progress bar</summary>
    /// <remarks>
    /// Calls internal_progress() with Invoke() for the case, if called from
    /// a thread routine.
    /// </remarks>
    /// <param name="perc">the progress value in percents</param>
    public void progress(int perc) {
      if (perc != this.m_progressval) {
        this.m_progressval = perc;
        this.Invoke(this.m_evprogress);
      }
    } // progress()

    /// <summary>The dictionary indexing is ready</summary>
    public void ready(string msg) {
      if (msg != null) MessageBox.Show(msg, Dict.res("Info"));
    } // ready()

    /// <summary>Handles button selecting the index file</summary>
    /// <remarks>Opens a file dialog box fo selecting the index file</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">e</param>
    private void onSelIdxFile(object sender, EventArgs e) {
      StringBuilder  sb;
      OpenFileDialog ofd;
      DirectoryInfo  di;
      FileInfo       fi;
      string         ext;

      fi = new FileInfo(this.m_idxfile.Text);
      di = fi.Directory;
      ext = fi.Extension;
      sb = new StringBuilder();
      if (!string.IsNullOrEmpty(ext)) {
        sb.Append(ext).Append("|*").Append(ext).Append('|');
      }
      sb.Append(Dict.res("All")).Append("|*.*");
      ofd = new OpenFileDialog();
      ofd.Filter = sb.ToString();
      ofd.FilterIndex = 1;
      if (di.Exists) ofd.InitialDirectory = di.FullName;
      if (ofd.ShowDialog() == DialogResult.OK) {
        this.m_idxfile.Text = ofd.FileName;
      }
    } // onLoadIdxFile()

    /// <summary>Handles button loading the dictionary file</summary>
    /// <remarks>
    /// Creates index from the dictionary file in m_dictfile.
    /// The resulting index file is created under the name from m_idxfile.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">e</param>
    private void onLoadDictFile(object sender, EventArgs e) {
      this.stop();
      this.m_dictimport = new DictImport(this,
                                         this.m_dictfile.Text,
                                         this.m_idxfile.Text);
      this.m_dictimport.start();
    } // onLoadDictFile()

    /// <summary>Handles button selecting the dictionary file</summary>
    /// <remarks>Opens a file dialog box for selecting the dictionary file</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">e</param>
    private void onSelDictFile(object sender, EventArgs e) {
      StringBuilder  sb;
      OpenFileDialog ofd;
      DirectoryInfo  di;
      FileInfo       fi;
      string         ext;

      fi = new FileInfo(this.m_dictfile.Text);
      di = fi.Directory;
      ext = fi.Extension;
      sb = new StringBuilder();
      if (!string.IsNullOrEmpty(ext)) {
        sb.Append(ext).Append("|*").Append(ext).Append('|');
      }
      sb.Append(Dict.res("All")).Append("|*.*");
      ofd = new OpenFileDialog();
      ofd.Filter = sb.ToString();
      ofd.FilterIndex = 1;
      if (di.Exists) ofd.InitialDirectory = di.FullName;
      if (ofd.ShowDialog() == DialogResult.OK) {
        this.m_dictfile.Text = ofd.FileName;
      }
    } // onSelDictFile()

    /// <summary>Handles closing the form</summary>
    /// <remarks>
    /// Forgets all done until now and closes the window. Shows a warning, if an
    /// index file has been loaded, because this method will discard any loaded
    /// indexes.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">e</param>
    private void onClose(object sender, EventArgs e) {
      this.stop();
      if (this.DialogResult == DialogResult.OK && this.m_cfg != null) {
        this.m_cfg.idxfile = this.m_idxfile.Text;
        this.m_cfg.dictfile = this.m_dictfile.Text;
      } // if
    } // onClose()
 
    /// <summary>Stops any background processes</summary>
    private void stop() {
      if (this.m_dictimport != null) {
        this.m_dictimport.Dispose();
        this.m_dictimport = null;
      }
    } // stop()

    /// <summary>Layouts the form</summary>
    private void InitializeComponent() {
      Label    lbl_idxfile, lbl_dictfile;
      Button   btn_selidxfile, btn_seldictfile, btn_loaddictfile,
               btn_cancel, btn_ok;
      Size     siz_but;

      this.SuspendLayout();
      this.AutoScroll = true;
      this.Size = new Size(260, 210);
      this.Controls.Clear();
      siz_but = new Size( 70, 20);
      lbl_idxfile = new Label();
      lbl_idxfile.Location = new Point(5, 5);
      lbl_idxfile.Text = Dict.res("Index file");
      this.m_idxfile.Location = new Point(5, 30);
      this.m_idxfile.Size = new Size(170, 30);
      lbl_dictfile = new Label();
      lbl_dictfile.Location = new Point(5, 60);
      lbl_dictfile.Text = Dict.res("Dictionary file");
      this.m_dictfile.Location = new Point(5, 85);
      this.m_dictfile.Size = new Size(100, 30);
      btn_selidxfile = new Button();
      btn_selidxfile.Location = new Point(175, 30);
      btn_selidxfile.Size = siz_but;
      btn_selidxfile.Text = Dict.res("Select");
      btn_seldictfile = new Button();
      btn_seldictfile.Location = new Point(105, 85);
      btn_seldictfile.Size = siz_but;
      btn_seldictfile.Text = Dict.res("Select");
      btn_loaddictfile = new Button();
      btn_loaddictfile.Location = new Point(175, 85);
      btn_loaddictfile.Size = siz_but;
      btn_loaddictfile.Text = Dict.res("Load");
      this.m_progress.Location = new Point(5, 120);
      this.m_progress.Size = new Size(240, 20);
      btn_ok = new Button();
      btn_ok.Location = new Point(40, 150);
      btn_ok.Size = siz_but;
      btn_ok.Text = Dict.res("OK");
      btn_ok.DialogResult = DialogResult.OK;
      btn_cancel = new Button();
      btn_cancel.Location = new Point(140, 150);
      btn_cancel.Size = siz_but;
      btn_cancel.Text = Dict.res("Cancel");
      btn_cancel.DialogResult = DialogResult.Cancel;
      this.Controls.Add(lbl_idxfile);
      this.Controls.Add(this.m_idxfile);
      this.Controls.Add(btn_selidxfile);
      this.Controls.Add(lbl_dictfile);
      this.Controls.Add(this.m_dictfile);
      this.Controls.Add(btn_seldictfile);
      this.Controls.Add(btn_loaddictfile);
      this.Controls.Add(this.m_progress);
      this.Controls.Add(btn_ok);
      this.Controls.Add(btn_cancel);
      this.Text = Dict.res("Config");
      this.ResumeLayout(false);
      btn_selidxfile.Click += this.onSelIdxFile;
      btn_seldictfile.Click += this.onSelDictFile;
      btn_loaddictfile.Click += this.onLoadDictFile;
    } // InitializeComponent()

    /// <summary>Initializes the form</summary>
    public CfgForm(DictCfg cfg) {
      this.m_cfg = cfg;
      this.m_dictimport = null;
      this.m_evprogress = new EventHandler(this.internal_progress);
      this.m_idxfile = new TextBox();
      this.m_dictfile = new TextBox();
      this.m_progress = new ProgressBar();
      this.m_progress.Minimum = 0;
      this.m_progress.Maximum = 100;
      this.m_progressval = 0;
      this.FormBorderStyle = FormBorderStyle.FixedDialog;
      this.Closing += this.onClose;
      this.InitializeComponent();
      if (cfg != null) {
        this.m_idxfile.Text = cfg.idxfile;
        this.m_dictfile.Text = cfg.dictfile;
      }
    } // CfgForm()

  } // class CfgForm

} // namespace dict

