using System;
using System.IO;
using System.Reflection;

namespace dict {

  /// <summary>Dictionary configuration</summary>
  public class DictCfg : IDisposable {

    /// <summary>Name of the configuration file</summary>
    /// <remarks>
    /// The name and directory of the configuration file is the same as
    /// the name and directory of the executable with the extension cfg.
    /// </remarks>
    private static string CONFIGFILE {
      get {
        string exepath;
        Uri    u;

        exepath = Assembly.GetExecutingAssembly().GetName().CodeBase;
        u = new Uri(exepath);
        if (u.IsFile) exepath = u.LocalPath;
        return Path.ChangeExtension(exepath, ".cfg");
      }
    } // CONFIGFILE

    /// <summary>The default index file name</summary>
    private const string DEF_IDXFILENAME = "dict.idx";

    /// <summary>The default dictionary file name</summary>
    private const string DEF_DICTFILENAME = "dict.txt";

    /// <summary>The default keyword</summary>
    private const string DEF_KEYWORD = "";

    /// <summary>Name of the index file</summary>
    private string m_idxfilename;

    /// <summary>Name of the dictionary file to use</summary>
    /// <remarks>
    /// Not necessarly the same name as in the index file itself.
    /// This is only the last entry in the dictionary file name input
    /// in the configuration form.
    /// </remarks>
    private string m_dictfilename;

    /// <summary>The last keyword queried in the dictionary</summary>
    private string m_keyword;

    /// <summary>Initializes the configuration from the CONFIGFILE</summary>
    public DictCfg() {
      this.m_idxfilename = DictCfg.DEF_IDXFILENAME;
      this.m_dictfilename = DictCfg.DEF_DICTFILENAME;
      this.m_keyword = DEF_KEYWORD;
      this.readCfg();
    } // DictCfg()

    /// <summary>Reads the configuration from m_cfgfilename</summary>
    private void readCfg() {
      StreamReader stream;
      string       idxfilename, dictfilename, keyword;

      stream = null;
      idxfilename = null;
      dictfilename = null;
      keyword = null;
      try {
        stream = new StreamReader(DictCfg.CONFIGFILE);
        idxfilename = stream.ReadLine();
        dictfilename = stream.ReadLine();
        keyword = stream.ReadLine();
      }
      catch(Exception) { }
      if (stream != null) {
        try { stream.Close(); } catch(Exception) {}
      }
      if (idxfilename != null) this.m_idxfilename = idxfilename;
      if (dictfilename != null) this.m_dictfilename = dictfilename;
      if (keyword != null) this.m_keyword = keyword;
    } // readCfg()

    /// <summary>Writes configuration into the CONFIGFILE</summary>
    private void writeCfg() {
      StreamWriter stream;

      stream = null;
      try {
        stream = new StreamWriter(DictCfg.CONFIGFILE);
        stream.WriteLine(this.m_idxfilename);
        stream.WriteLine(this.m_dictfilename);
        stream.WriteLine(this.m_keyword);
      }
      catch(Exception) {}
      if (stream != null) {
        try { stream.Close(); } catch(Exception) {}
      }
    } // writeCfg()

    /// <summary>Writes configuration into the CONFIGFILE</summary>
    public void Dispose() {
      this.writeCfg();
    } // Dispose()

    /// <summary>Name of the index file</summary>
    public string idxfile {
      get { return this.m_idxfilename; }
      set { this.m_idxfilename = value; }
    } // idxfile

    /// <summary>Name of the dictionary file to use</summary>
    /// <remarks>
    /// Not necessarly the same name as in the index file itself.
    /// This is only the last entry in the dictionary file name input
    /// in the configuration form.
    /// </remarks>
    public string dictfile {
      get { return this.m_dictfilename; }
      set { this.m_dictfilename = value; }
    } // dictfile

    /// <summary>The last keyword queried in the dictionary</summary>
    public string keyword {
      get { return this.m_keyword; }
      set { this.m_keyword = value; }
    } // keyword

  } // class DictCfg

} // namespace dict

