using System;
using System.IO;
using System.Text;

namespace dict {

  /// <summary>Reference to a dictionary entry in the dictionary file</summary>
  /// <remarks>
  /// Contains a start position and the length of the whole dictionary entry
  /// in a file. For example for a file obtained from dict.cc it is the position
  /// of the begin of the line in the file and the length of the file in bytes.
  /// </remarks>
  public class DictEntryRef {

    /// <summary>Encoding for the dictionary file, Windows 1252</summary>
    public static Encoding DICT_ENCODING = Encoding.Default;

    /// <summary>The position of the begin of the entry</summary>
    private int m_beg;

    /// <summary>The length of the entry in bytes</summary>
    private short  m_len;

    /// <summary>Initializes the entry for the given positions</summary>
    /// <param name="from">
    /// Position of the begin of the entry in the dictionary file
    /// </param>
    /// <param name="to">
    /// Position just after the end of the entry in the dictionary file
    /// </param>
    public DictEntryRef(long from, long to) {
      this.m_beg = (int) from;
      this.m_len = (short) (to - from);
      if (this.m_len < 0) this.m_len = 0;
    } // DictEntryRef()

    /// <summary>The position of the begin of the entry</summary>
    public int beg {
      get {
        return this.m_beg;
      }
    } // beg

    /// <summary>The length of the entry in bytes</summary>
    public short len {
      get {
        return this.m_len;
      }
    } // len

    /// <summary>Reads entry from the dictionary file</summary>
    /// <remarks>
    /// Positions the string at the position of the beginning of the
    /// dictionary entry m_beg and reads m_len bytes from it.
    /// </remarks>
    /// <param name="input">The stream from which to read</param>
    /// <returns>The dictionary entry converted from dict encoding to string</returns>
    public string read(Stream input) {
      byte[] b;
      string retval;
      long   oldpos;
      int    len;

      if (this.m_len > 0) { 
        b = new byte[this.m_len];
        oldpos = input.Position;
        input.Position = (long) this.m_beg;
        len = input.Read(b, 0, b.Length - 1);
        retval = DictEntryRef.DICT_ENCODING.GetString(b, 0, len);
        input.Position = oldpos;
      }
      else 
        retval = string.Empty;
      return retval;
    } // read()

    /// <summary>String representation of the object</summary>
    /// <returns>
    /// The string representation of the object as two integer numbers:
    /// the position of the begin of the entry m_beg and the length of the
    /// entry in bytes m_len, separated with a whitespace.
    /// </returns>
    public override string ToString() {
      StringBuilder sb;

      sb = new StringBuilder();
      sb.Append(this.m_beg).Append(' ').Append(this.m_len);
      return sb.ToString();
    } // ToString()

  } // class DictEntryRef

} // namespace dict

