using System;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace dict {

  /// <summary>Main form of the dictionary</summary>
  public class DictForm : Form {

    /// <summary>Input box for the keyword to search</summary>
    private TextBox m_keyword;

    /// <summary>The answers for keyword</summary>
    private TextBox m_answer;

    /// <summary>The dictionary index</summary>
    private DictIndex m_dictindex;

    /// <summary>The program condfiguration</summary>
    private DictCfg m_cfg;

    /// <summary>
    /// Common method for showing exceptions occured in this form.
    /// </summary>
    /// <param name="aTitle">the title of the exception</param>
    /// <param name="anExcp">the exception itself</param>
    private void printExcp(string aTitle, Exception anExcp) {
      StringBuilder sb;

      if (anExcp != null) {
        sb = new StringBuilder();
        if (aTitle != null) sb.Append(aTitle).Append(Utils.newline);
        sb.Append(anExcp.ToString());
        MessageBox.Show(sb.ToString(), Dict.res("Error"));
      }
    } // printExcp()

    /// <summary>Event method called if the form is closed</summary>
    /// <remarks>
    /// Cleans up everything before leaving.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onClose(object sender, EventArgs e) {
      if (this.m_cfg != null) {
        this.m_cfg.Dispose();
      }
    } // onClose()

    /// <summary>Called if the form gets resized</summary>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    /// <seealso cref="resizeControls">resizeControls</seealso>
    private void onResize(object sender, EventArgs e) {
      this.resizeControls();
    } // onResize()

    /// <summary>Called if the user enters keyword input box</summary>
    /// <remarks>
    /// Clears the content of the control. On Windows CE activates SIP.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    /// <seealso cref="Utils.sip">Utils.sip</seealso>
    private void onKeywordGotFocus(object sender, EventArgs e) {
      Utils.sip(true);
      this.m_keyword.Text = string.Empty;
    } // onKeywordGotFocus()

    /// <summary>Handles menu item dict/exit</summary>
    /// <remarks>Exits the application.</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onMenuExit(object sender, EventArgs e) {
      this.Close();
    } // onMenuExit()

    /// <summary>Handles menu item dict/config</summary>
    /// <remarks>Starts the configuration window</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onMenuCfg(object sender, EventArgs e) {
      CfgForm frm;

      try {
        frm = new CfgForm(this.m_cfg);
        if (frm.ShowDialog() == DialogResult.OK) {
          if (this.m_dictindex != null) {
            this.m_dictindex.Dispose();
            this.m_dictindex = null;
          }
          this.m_dictindex = DictIdxRdForm.rdIndex(this.m_cfg.idxfile);
        }
      }
      catch(Exception excp) {
        this.printExcp(Dict.res("Configuration error"), excp);
      }
    } // onMenuCfg()

    /// <summary>Handles menu item search/reg. expr.</summary>
    /// <remarks>Starts the reg. expr. based search for the keyword.</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onMenuSearchRE(object sender, EventArgs e) {
      StringBuilder sb;
      string[]      res;
      int           len;

      if (this.m_dictindex != null) {
        this.m_answer.Text = string.Empty;
        res = this.m_dictindex.getre(this.m_keyword.Text);
        if (res != null) {
          len = 0;
          foreach(string s in res) { 
            if (s != null) {
              len += s.Length;
              len += Utils.newline.Length;
            }
          }
          if (len > 0) {
            this.m_cfg.keyword = this.m_keyword.Text;
            sb = new StringBuilder(len);
            foreach(string s in res) {
              if (s != null) sb.Append(s).Append(Utils.newline);
            }
            this.m_answer.Text = sb.ToString();
            this.m_answer.Focus();
          }
        }
      }
    } // onMenuSearchRE()

    /// <summary>Handles menu item search/exact</summary>
    /// <remarks>Starts the exact search of the keyword.</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onMenuSearchExact(object sender, EventArgs e) {
      StringBuilder sb;
      string[]      res;
      int           len;

      if (this.m_dictindex != null) {
        this.m_answer.Text = string.Empty;
        res = this.m_dictindex.get(this.m_keyword.Text);
        if (res != null) {
          len = 0;
          foreach(string s in res) { 
            if (s != null) {
              len += s.Length;
              len += Utils.newline.Length;
            }
          }
          if (len > 0) {
            this.m_cfg.keyword = this.m_keyword.Text;
            sb = new StringBuilder(len);
            foreach(string s in res) {
              if (s != null) sb.Append(s).Append(Utils.newline);
            }
            this.m_answer.Text = sb.ToString();
            this.m_answer.Focus();
          }
        }
      }
    } // onMenuSearchExact()

    /// <summary>
    /// Estimates the size of  controls basing on the current form size
    /// </summary>
    private void resizeControls() {
      Size siz;
      int  h, w;

      siz = this.Size;
      h = siz.Height;
      w = siz.Width;
      this.m_keyword.Location = new Point(5, 5);
      this.m_keyword.Size = new Size(w - 20, 30);
      this.m_answer.Location = new Point(5, 40);
      this.m_answer.Size = new Size(w - 20, h - 120);
    } // resizeControls()

    /// <summary>Layout of the form</summary>
    private void InitializeComponent() {
      MainMenu menu;
      MenuItem menuDict, menuExit, menuCfg, 
               menuSearch, menuSearchRE, menuSearchExact;

      this.SuspendLayout();
      this.AutoScroll = true;
      this.Controls.Clear();
      this.resizeControls();
      this.Controls.Add(this.m_keyword);
      this.Controls.Add(this.m_answer);
      menuExit = new MenuItem();
      menuExit.Text = Dict.res("Exit");
      menuExit.Click += this.onMenuExit;
      menuCfg = new MenuItem();
      menuCfg.Text = Dict.res("Config");
      menuCfg.Click += this.onMenuCfg;
      menuDict = new MenuItem();
      menuDict.Text = Dict.res("Dictionary");
      menuDict.MenuItems.Add(menuCfg);
      menuDict.MenuItems.Add(menuExit);
      menuSearchRE = new MenuItem();
      menuSearchRE.Text = Dict.res("Reg. Expr.");
      menuSearchRE.Click += this.onMenuSearchRE;
      menuSearchExact = new MenuItem();
      menuSearchExact.Text = Dict.res("Exact");
      menuSearchExact.Click += this.onMenuSearchExact;
      menuSearch = new MenuItem();
      menuSearch.Text = Dict.res("Search");
      menuSearch.MenuItems.Add(menuSearchExact);
      menuSearch.MenuItems.Add(menuSearchRE);
      menu = new MainMenu();
      menu.MenuItems.Add(menuDict);
      menu.MenuItems.Add(menuSearch);
      this.Menu = menu;
      this.Text = Dict.res("Dictionary");
      this.Name = "Dictionary";
      this.ResumeLayout(false);
    } // InitializeComponent()

    /// <summary>Initializes the form</summary>
    public DictForm() {
      Stream str;
      Icon   ico;

      this.m_dictindex = null;
      this.m_cfg = new DictCfg();
      this.m_keyword = new TextBox();
      this.m_keyword.Text = this.m_cfg.keyword;
      this.m_answer = new TextBox();
      this.m_answer.Multiline = true;
      this.m_answer.ReadOnly = true;
      this.m_answer.WordWrap = false;
      this.m_answer.ScrollBars = ScrollBars.Vertical | ScrollBars.Horizontal;
      this.Closing += this.onClose;
      this.Resize += this.onResize;
      this.m_keyword.GotFocus += this.onKeywordGotFocus;
      this.InitializeComponent();
      try {
        str = Assembly.GetExecutingAssembly().GetManifestResourceStream("dict.ico");
        ico = new Icon(str);
        str.Close();
        this.Icon = ico;
      }
      catch(Exception excp) {
        this.printExcp("ctor", excp);
      }
      try {
        if (File.Exists(this.m_cfg.idxfile)) 
          this.m_dictindex = DictIdxRdForm.rdIndex(this.m_cfg.idxfile);
      }
      catch(Exception excp) {
        this.printExcp(Dict.res("Initialization error"), excp);
      }
    } // DictForm()
  } // class DictForm

} // namespace dict

