using System;
using System.Drawing;
using System.Windows.Forms;

namespace dict {

  /// <summary>Form showing the progress of dictionary index reading</summary>
  /// <remarks>
  /// In GUI environments use DictIdxRdForm.rdIndex() rather that to
  /// instantiate the dictionary index object DictIndex directly.
  /// </remarks>
  /// <example>
  /// The following example creates a dictionary index object from the 
  /// index file <c>c:\temp\dict.idx</c> and translates the English word
  /// <c>query</c> using this dictionary index:
  /// <code lang="C#">
  /// using dict;
  /// public class Test {
  ///   private const string INDEXFILE = @"c:\temp\dict.idx";
  ///   public static void Main() {
  ///     DictIndex idx;
  ///     string[]  answer;
  ///
  ///     idx = DictIdxRdForm.rdIndex(Test.INDEXFILE);
  ///     answer = idx.get("query");
  ///     if (answer != null) {
  ///       foreach(string s in answer) Console.WriteLine(s);
  ///     }
  ///   }
  /// }
  /// </code>
  /// </example>
  /// <seealso cref="DictIndex">DictIndex</seealso>
  /// <seealso cref="DictImport.saveIdx">DictImport.saveIdx</seealso>
  public class DictIdxRdForm : Form, IProgress {

    /// <summary>The progress value to set in the progress bar</summary>
    private int m_progressval;

    /// <summary>The dictionary index read in this form</summary>
    private DictIndex m_index;

    /// <summary>The progress bar for showing the progress</summary>
    private ProgressBar m_progress;

    /// <summary>internal_progress() as event</summary>
    /// <seealso cref="internal_progress">internal_progress</seealso>
    private EventHandler m_evprogress;

    /// <summary>Event for closing the form with Invoke()</summary>
    /// <seealso cref="internal_close">internal_close</seealso>
    private EventHandler m_evclose;

    /// <summary>Prepares the form</summary>
    /// <remarks>
    /// Use read() just after the constructor in order to start reading the
    /// index contents.
    /// </remarks>
    private DictIdxRdForm() {
      this.m_progressval = 0;
      this.m_progress = new ProgressBar();
      this.m_progress.Minimum = 0;
      this.m_progress.Maximum = 100;
      this.m_evprogress = new EventHandler(this.internal_progress);
      this.m_evclose = new EventHandler(this.internal_close);
      this.FormBorderStyle = FormBorderStyle.FixedDialog;
      this.Closing += this.onClose;
      this.InitializeComponent();
      this.m_index = null;
    } // DictIdxRdForm()

    /// <summary>Reads the index contents from the file</summary>
    private void read(string anIdxFileName) {
      this.m_index = new DictIndex(anIdxFileName, this);
    } // read()

    /// <summary>Places the controls on the form</summary>
    private void InitializeComponent() {
      Label lbl;

      this.SuspendLayout();
      this.Text = Dict.res("Reading index file");
      this.Size = new Size(220, 90);
      this.Controls.Clear();
      lbl = new Label();
      this.m_progress.Location = new Point(5, 5);
      this.m_progress.Size = new Size(200, 30);
      lbl.Location = new Point(5, 40);
      lbl.Size = new Size(200, 30);
      lbl.Text = Dict.res("Please wait ...");
      this.Controls.Add(this.m_progress);
      this.Controls.Add(lbl);
      this.ResumeLayout(false);
    } // InitializeComponent()

    /// <summary>Returns the index read in this form</summary>
    private DictIndex index {
      get {
        return this.m_index;
      }
    } // index

    /// <summary>Called if the form becomes closed</summary>
    /// <remarks>Stops index reading thread</remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    private void onClose(object sender, EventArgs e) {
    } // onClose()

    /// <summary>Shows progress in the progress bar</summary>
    /// <remarks>
    /// Internal meaning: called from progress() using Invoke().
    /// Sets the progress bar value in m_progress from m_progressval.
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    /// <seealso cref="m_evprogress">m_evprogress</seealso>
    /// <seealso cref="progress">progress</seealso>
    private void internal_progress(object sender, EventArgs e) {
      this.m_progress.Value = this.m_progressval;
    } // internal_progress()

    /// <summary>Closes the form</summary>
    /// <remarks>
    /// Internal meaning: called from ready() using Invoke().
    /// </remarks>
    /// <param name="sender">ignored</param>
    /// <param name="e">ignored</param>
    /// <seealso cref="m_evclose">m_evclose</seealso>
    /// <seealso cref="ready">ready</seealso>
    private void internal_close(object sender, EventArgs e) {
      this.Close();
    } // internal_progress()

    /// <summary>Shows progress in the progress bar</summary>
    /// <param name="perc">The percentage progress value</param>
    public void progress(int perc) {
      if (perc != this.m_progressval) {
        this.m_progressval = perc;
        try {
          this.Invoke(this.m_evprogress);
        }
        catch(Exception) {}
      }
    } // progress()

    /// <summary>Closes the form, evtl. shows the given message</summary>
    /// <remarks>
    /// Called, if the dictionary reading process is ready.
    /// Closing form is performed through Invoke and m_evclose, else
    /// fatal exceptions are generated on PDA.
    /// </remarks>
    /// <param name="msg">
    /// Message to show prior to closing the form. If null, nothing will be
    /// shown at all.
    /// </param>
    public void ready(string msg) {
      if (msg != null) MessageBox.Show(msg, Dict.res("Info"));
      this.Invoke(this.m_evclose);
    } // ready()

    /// <summary>Joins the index reading thread</summary>
    private void stop() {
      if (this.m_index != null) this.m_index.stop();
    } // Dispose()

    /// <summary>Reads dictionary index from the given file</summary>
    /// <remarks>
    /// Opens the form reading the index file and returns the index read.
    /// </remarks>
    /// <param name="anIdxFileName">the name of the index file</param>
    /// <returns>the dictionary index created from the file</returns>
    public static DictIndex rdIndex(string anIdxFileName) {
      DictIdxRdForm frm;

      frm = new DictIdxRdForm();
      frm.read(anIdxFileName);
      frm.ShowDialog();
      frm.stop();
      return frm.index;
    } // rdIndex()
  } // class DictIdxRdForm

} // namespace dict

