using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading;

namespace dict {

  /// <summary>Used for dictionary index file creation</summary>
  /// <remarks>
  /// Use this class in order to create a dictionary index file 
  /// (<A HREF="http://www1.dict.cc/translation_file_request.php">http://www1.dict.cc/translation_file_request.php</A>).
  /// The index creation, started in the method start() in createproc()
  /// uses the dictionary parse DictParser, which extracts the keywords from
  /// the dictionary file. Each keyword is checked using the regular expression
  /// MUSTMATCH, if it is suitable for use as the dictionary keyword. Generally
  /// only keywords consistiong of at least two letters are accepted, remaining
  /// keywords are ignored. The keywords are then collected as keys in a local
  /// map, the values for each keys are offsets and lengths in bytes of the
  /// locations in the dict.cc dictionary file, where the keywords are
  /// explained. The dictionary file from dict.cc is in a huge part a free text
  /// file, so than many keywords, like for example the English <c>the</c> or
  /// <c>of</c> may easyly have several thousands explanations. In order to
  /// limit the amount of translations to a human acceptable level, the
  /// class accepts max. MAXENTRIES (150) explanations for one keyword, the
  /// remaining explanations are slightly ignored. See also the description
  /// of the class DictParser for more details of the expected format of
  /// the dictionary file and the description of the method
  /// DictImport.saveIdx() for more details on the format of the dictionary
  /// index file created here. The index files created in DictImport.saveIdx()
  /// may be used for instantationg the dictionary index objects DictIndex
  /// used for querying the dictionary.
  /// </remarks>
  /// <example>
  /// The following example creates a dictionary index file from a dictionary
  /// file obtained from dict.cc:
  /// <c>c:\temp\engdeu.idx</c> from a dictionary file <c>c:\temp\engdeu.txt</c>
  /// obtained from dict.cc:
  /// <code lang="C#">
  /// using dict;
  /// using System;
  /// public class Import : IProgress {
  ///   public void progress(int perc) {
  ///     Console.WriteLine("Imported {0} percents", perc);
  ///   }
  ///   public void ready(string message) {
  ///     if (message != null) Console.WriteLine(message);
  ///     Console.WriteLine("Ready, press ENTER to exit");
  ///   }
  ///   public Import(string dictfile, string idxfile) {
  ///     
  ///     DictImport imp;
  ///     imp = new DictImport(this, dictfile, idxfile);
  ///     imp.start();
  ///     Console.WriteLine("Wait until ready or press ENTER to break");
  ///     Console.ReadLine();
  ///     imp.stop();
  ///     if (imp.lastexcp != null) Console.Writeline(imp.lastexcp);
  ///   }
  ///   public static void Main() {
  ///     new Import(@"c:\temp\engdeu.txt", @"c:\temp\engdeu.idx");
  ///   }
  /// }
  /// </code>
  /// </example>
  /// <seealso cref="DictParser">DictParser</seealso>
  /// <seealso cref="DictIndex">DictIndex</seealso>
  public class DictImport : IDisposable, IDictParserHandler {

    /// <summary>Reg. expr. which each index keyword must match</summary>
    /// <remarks>
    /// Index keywords not matching this reg. expr. are assumed to be garbage
    /// and they're not included in the index.
    /// </remarks>
    private static readonly Regex MUSTMATCH = new Regex(@"^\p{L}(\p{L}|-)*\p{L}$",
                                                        RegexOptions.Compiled);

    /// <summary>Maximum number of entries per index</summary>
    public const int MAXENTRIES = 150;

    /// <summary>The index</summary>
    /// <remarks>
    /// Key are the keywords, eg. English keywords in an English-German
    /// dictionary. The values are arrays of DictEntryRef objects, referencing
    /// the whole entries in the dictionary file describing the keywords.
    /// </remarks>
    private Hashtable m_index;

    /// <summary>Name of the dictionary file to import</summary>
    private string m_dictfilename;

    /// <summary>Name of the resulting index file</summary>
    private string m_idxfilename;

    /// <summary>The length of the dictionary file</summary>
    private long m_dictfilelen;

    /// <summary>Handler for showing progress of the import process</summary>
    private IProgress m_progress_handler;

    /// <summary>Thread creating the index</summary>
    private Thread m_thr;

    /// <summary>Last exception occured during processing</summary>
    private Exception m_lastexcp;

    /// <summary>If the process should continue</summary>
    /// <remarks>true for continue, false for interrupted by the user</remarks>
    private bool m_continue;

    /// <summary>Initializes the creation of the dictionary index</summary>
    /// <param name="aProgressHandler">
    /// handler for showing the progress of the import process
    /// </param>
    /// <param name="aDictFileName">
    /// name of the dictionary file for which to create the index
    /// </param>
    /// <param name="anIdxFileName">
    /// name of the resulting index file
    /// </param>
    public DictImport(IProgress aProgressHandler,
                      string    aDictFileName,
                      string    anIdxFileName) {
      FileInfo fi;

      this.m_lastexcp = null;
      try {
        this.m_index = new Hashtable();
        this.m_progress_handler = aProgressHandler;
        this.m_dictfilename = aDictFileName;
        this.m_idxfilename = anIdxFileName;
        fi = new FileInfo(aDictFileName);
        this.m_dictfilelen = fi.Length;
        this.m_thr = null;
        this.m_continue = false;
      }
      catch(Exception excp) {
        this.m_lastexcp = excp;
      }
    } // DictImport()

    /// <summary>Disposes the object before beeing destroyed</summary>
    ~DictImport() {
       this.Dispose();
    } // ~DictImport()

    /// <summary>Disposes the object</summary>
    public void Dispose() {
      this.stop();
      if (this.m_index != null) {
        this.m_index.Clear();
        this.m_index = null;
      }
      this.m_dictfilename = null;
      this.m_idxfilename = null;
      this.m_progress_handler = null;
      this.m_dictfilelen = 0;
    } // Dispose()

    /// <summary>Stops the index creation process</summary>
    public void stop() {
      this.m_continue = false;
      if (this.m_thr != null) {
        this.m_thr.Join();
        this.m_thr = null;
      }
    } // stop()

    /// <summary>Starts the index creation process</summary>
    public void start() {
      this.stop();
      this.m_thr = new Thread(this.createproc);
      this.m_thr.IsBackground = true;
      this.m_continue = true;
      this.m_thr.Start();
    } // start()

    /// <summary>Thread routine creating the dictionary index</summary>
    private void createproc() {
      string     msg;
      Stream     dictfile;
      DictParser dp;

      dictfile = null;
      if (this.m_lastexcp == null) {
        try {
          this.m_index.Clear();
          dictfile = new FileStream(this.m_dictfilename,
                                    FileMode.Open,
                                    FileAccess.Read,
                                    FileShare.None);
          dp = new DictParser(this);
          dp.read(dictfile);
          DictImport.saveIdx(this.m_index,
                             this.m_dictfilename,
                             this.m_idxfilename,
                             this.m_progress_handler);
        }
        catch(Exception excp) {
          if (this.m_lastexcp == null) this.m_lastexcp = excp;
        }
        if (dictfile != null) {
          try { dictfile.Close(); } catch(Exception) { }
        }
      } // if (this.m_lastexcp == null) 
      if (this.m_progress_handler != null && this.m_continue) {
        if (this.lastexcp != null)
          msg = this.lastexcp.ToString();
        else
          msg = string.Format(Dict.res("Successfully created index {0} from dictionary {1}"),
                              this.m_idxfilename,
                              this.m_dictfilename);
        this.m_progress_handler.ready(msg);
      }
    } // createproc()

    /// <summary>Adds the given dictionary index entry</summary>
    /// <remarks>
    /// Adds a new DictEntryRef object build from the given from and to
    /// offsets to the array of dictionary entry references for the given
    /// index. 
    /// The method does absolutely nothing on disposed objects.
    /// The does also nothing, if the given index already references at
    /// least MAXENTRIES entries. This is because some popular words like
    /// of, in, the may be contained in almost all entries of the dictionary.
    /// </remarks>
    /// <param name="index">the index keyword</param>
    /// <param name="from">
    /// the position in the dictionary file, at which the dictionary entry
    /// for the index starts
    /// </param>
    /// <param name="to">
    /// the position in the dictionary file just after the end of the
    /// dictionary entry for the index
    /// </param>
    /// <returns>
    /// True, if the process should continue or false if it was interrupted
    /// by the user
    /// </returns>
    public bool addIndex(string index, long from, long to) {
      ArrayList al;
      int       progval;

      if (this.m_index != null &&
          index != null &&
          DictImport.MUSTMATCH.IsMatch(index)) {
        if (this.m_index.ContainsKey(index))
          al = (ArrayList) this.m_index[index];
        else
          al = new ArrayList();
        if (al.Count < DictImport.MAXENTRIES) {
          al.Add(new DictEntryRef(from, to));
          this.m_index[index] = al;
        }
        if (this.m_progress_handler != null && this.m_dictfilelen != 0) {
          progval = (int) (to * 100 / this.m_dictfilelen);
          if (this.m_continue) this.m_progress_handler.progress(progval);
        }
      }
      return this.m_continue;
    } // addIndex()

    /// <summary>Creates an index file from the index created anIndex</summary>
    /// <remarks>
    /// The index file created is a binary file (System.IO.BinaryWriter)
    /// having the following format:
    /// <list type="bullet">
    /// <item>
    /// Number of index keywords n_keywords, Int32.
    /// </item>
    /// <item>
    /// Length of the name of the dict.cc dictionary file in bytes, Int32.
    /// </item>
    /// <item>
    /// Name of the dict.cc dictionary file, bytes in encoding
    /// DictEntryRef.DICT_ENCODING.
    /// </item>
    /// <item>
    /// n_keywords entries having the following format:
    ///  <list type="bullet">
    ///  <item>Length of the index keyword in bytes, Int32</item>
    ///  <item>The index keyword, bytes in encoding DictEntryRef.DICT_ENCODING</item>
    ///  <item>Starting index of the first DictEntryRef for the keyword, Int32</item>
    ///  <item>Number of DictEntryRef entries for the keyword, Int16</item>
    ///  </list>
    /// </item>
    /// <item>
    /// The biggest part on the end of the index file are DictEntryRef entries
    /// each consisting of:
    ///  <list type="bullet">
    ///  <item>
    ///  Begin position of the keyword description in the dict.cc dictionary
    ///  file, Int32.
    ///  </item>
    ///  <item>
    ///  Number of bytes for the keyword description in the dict.cc dictionary
    ///  file, Int16.
    ///  </item>
    ///  </list>
    /// The number and sequence of the DictEntryRef entries in the file 
    /// according to the data for the index keyword, ie. first come entries
    /// for the keyword1, then for keyword2 etc.
    /// </item>
    /// </list>
    /// </remarks>
    /// <param name="anIndex">
    /// The index. The keys are index keywords as strings, eg. English keywords
    /// in the English-German dictionary. The values are ArrayLists of
    /// DictEntryRef objects with the references to the keyword descriptions
    /// in the dictionary file aDictFileName.
    /// </param>
    /// <param name="aDictFileName">
    /// The name or path to the dict.cc dictionary file.
    /// </param>
    /// <param name="anIdxFileName">
    /// Name or path to the index file to create.
    /// </param>
    /// <param name="aProgressHandler">
    /// Handler for showing progress of the storing operation.
    /// </param>
    /// <seealso cref="DictIndex">DictIndex</seealso>
    private static void saveIdx(Hashtable anIndex,
                                string    aDictFileName,
                                string    anIdxFileName,
                                IProgress aProgressHandler) {
      DictEntryRef der;
      BinaryWriter w;
      ICollection  keys;
      Exception    e;
      ArrayList    al;
      byte[]       barr;
      int          num, num2, i, perc, percold, index, cnt;

      w = null;
      e = null;
      try {
        w = new BinaryWriter(File.OpenWrite(anIdxFileName));
        if (anIndex != null) {
          keys = anIndex.Keys;
          num = anIndex.Count;
        }
        else {
          keys = null;
          num = 0;
        }
        w.Write(num);
        barr = DictEntryRef.DICT_ENCODING.GetBytes(aDictFileName);
        w.Write(barr.Length); w.Write(barr);
        if (num > 0) {
          num2 = num + num;
          i = 0; percold = 0; index = 0;
          foreach(object key in keys) {
            al = (ArrayList) anIndex[key];
            if (al != null) {
              barr = DictEntryRef.DICT_ENCODING.GetBytes(key.ToString());
              w.Write(barr.Length); w.Write(barr);
              cnt = al.Count;
              w.Write(index);
              w.Write(cnt);
              index += cnt;
            }
            i++;
            perc = (int) (i * 100 / num2);
            if (perc != percold) {
              percold = perc;
              if (aProgressHandler != null) aProgressHandler.progress(perc);
            }
          } // foreach key
          foreach(object key in keys) {
            al = (ArrayList) anIndex[key];
            if (al != null) {
              foreach(object key1 in al) {
                der = (DictEntryRef) key1;
                w.Write(der.beg);
                w.Write(der.len);
              }
            }
            i++;
            perc = (int) (i * 100 / num2);
            if (perc != percold) {
              percold = perc;
              if (aProgressHandler != null) aProgressHandler.progress(perc);
            }
          }
        } // if (num > 0) 
      }
      catch(Exception excp) { e = excp; }
      if (w != null) { 
        try {
          w.Close();
        }
        catch(Exception) {}
      }
      if (e != null) throw e;
    } // dump()

    /// <summary>Last exception occured in the class, if any</summary>
    /// <remarks>Returns null, if there were no exceptions</remarks>
    public Exception lastexcp {
      get {
        return this.m_lastexcp;
      }
    } // lastexcp
  } // class DictImport
} // namespace dict

