using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading;

namespace dict {

  /// <summary>Dictionary index</summary>
  /// <remarks>
  /// <p>
  /// The dictionary index contains a map of keywords to locations
  /// in the dict.cc dictionary file describing the keywords. The keywords
  /// are for example English keywords in the English-German dictionary.
  /// The location of the description of the keyword in the dict.cc dictionary
  /// file is described in a DictEntryRef entry. Each keyword is mapped to
  /// an array of DictEntryRef entries, because one keyword is usually 
  /// described by many lines in the dictionary file. The best example could
  /// be the description of English keywords like <c>of</c> or <c>the</c>,
  /// which may have
  /// several thousands corresponding descriptions (though the number of
  /// such description is limited in DictImport to DictImport.MAXENTRIES,
  /// usually a max. human acceptable number like 150 or so). The whole index
  /// is kept in the memory of the computer/device, what can be problematic
  /// on small PDA devices with a limited amount of memory. Therefore the
  /// internal memory representation of the index is more compilated.
  /// The map of the index keeps only keywords as keys, the values for each
  /// keyword are two integers: the starting index in a huge array of
  /// DictEntryRef entries and the number of these entries in the array.
  /// While looking for keywords both integers are retrieved from the memory
  /// and the array of DictEntryRef entries is then retrieved from the file.
  /// Using this array the program accesses then the dict.cc dictionary file
  /// and reads the keyword descriptions from there. Thus searching may be
  /// also performed on limited memory devices, though few megabytes
  /// of RAM is required for the program to run.
  /// </p>
  /// <p>
  /// The format of the index file from which the index is build is described
  /// by the method DictImport.saveIdx. Generally it consists of the map
  /// keyword - two integers on the top of the file. This map is read into
  /// the memory. Then the file contains the huge array of DictEntryRef
  /// entries. While reading the index file the class remembers the position
  /// of the beginning of this huge array in the file and then it recomputes
  /// the location of the DictEntryRef entries relatively to this position
  /// assuming, that each DictEntryRef entry occupies 6 bytes (one Int32 and
  /// one Int16) in it.
  /// </p>
  /// <p>
  /// In GUI environments use DictIdxRdForm.rdIndex() rather than this class
  /// directly in order to create a dictionary index. The method creates a
  /// nice form with a progress bar while reading the index file. Reading
  /// of the index file on slow PDAs may take few (tenths) of seconds, so that
  /// showing a progress bar during the reading process is a must.
  /// </p>
  /// </remarks>
  /// <seealso cref="DictImport">DictImport</seealso>
  /// <seealso cref="DictIdxRdForm">DictIdxRdForm</seealso>
  public class DictIndex : IDisposable {

    /// <summary>Thread reading dictionary index from the file</summary>
    private Thread m_thr;

    /// <summary>The name of the dictionary file</summary>
    private string m_dictfilename;

    /// <summary>The name of the index file</summary>
    /// <remarks>
    /// This is the name of the file to read the dictionary index
    /// from in readObj(). This file is also always opened while looking
    /// for the huge array of DictEntryRef entries.
    /// </remarks>
    private string m_filename;

    /// <summary>The dictionary index itself</summary>
    /// <remarks>
    /// Contains keywords as keys and two integers as values. The first
    /// integer is the starting index in the array of DictEntryRef entries,
    /// the second integer is the number of DictEntryRef entries for
    /// the keyword.
    /// </remarks>
    private Hashtable m_index;

    /// <summary>For showing progress of index file reading or creation</summary>
    private IProgress m_progresshandler;

    /// <summary>Reader for the index file for reading array entries</summary>
    private BinaryReader m_indexfilereader;

    /// <summary>For reading the dict.cc dictionary file</summary>
    private Stream m_dictfilestream;

    /// <summary>
    /// The position of the array of DictEntryRef entries in the index file
    /// </summary>
    /// <remarks>
    /// This is the position of the DictEntryRef entry having index 0 in the
    /// file m_filename. The position of the entry with index n can be computed
    /// as m_arraybegin + 6 * n.
    /// </remarks>
    private long m_arraybegin;

    /// <summary>If false, then the reading thread has been interrupted</summary>
    private bool m_goon;

    /// <summary>Reads index from the given index file</summary>
    /// <remarks>
    /// Reading is performed in the background thread.
    /// The format of the index file is described by DictImport.saveIdx().
    /// The constructor initializes an empty dictionary index and starts
    /// readObj() in a background thread in order to read the index file.
    /// Reading can be always interrupted using stop(), but in this case
    /// the dictionary index object is left in an inconsistent state and it
    /// is unusable.
    /// </remarks>
    /// <param name="anIdxFileName">
    /// The name of the index file from which to read the dictionary index
    /// </param>
    /// <param name="aProgressHandler">
    /// For showing progress of the index reading process
    /// </param>
    /// <seealso cref="DictImport.saveIdx">DictImport.saveIdx</seealso>
    public DictIndex(string    anIdxFileName,
                     IProgress aProgressHandler) {
      this.m_dictfilestream = null;
      this.m_indexfilereader = null;
      this.m_index = null;
      this.m_dictfilename = string.Empty;
      this.m_filename = anIdxFileName;
      this.m_progresshandler = aProgressHandler;
      this.m_arraybegin = -1;
      this.m_goon = true;
      this.m_thr = new Thread(this.readObj);
      this.m_thr.Start();
    } // DictIndex()

    /// <summary>Releases the object</summary>
    /// <remarks>Calls Dispose()</remarks>
    /// <seealso cref="Dispose">Dispose</seealso>
    ~DictIndex() {
      this.Dispose();
    }
    /// <summary>Stops the reading thread</summary>
    public void stop() {
      this.m_goon = false;
      if (this.m_thr != null) {
        this.m_thr.Join();
        this.m_thr = null;
      }
    } // stop()

    /// <summary>Translates the given keyword</summary>
    /// <param name="keyword">The keyword to translate</param>
    /// <returns>
    /// Array of dictionary entries for the keyword or null if not found
    /// </returns>
    public string[] get(string keyword) {
      string[]       retval;
      DictEntryRef   der;
      int[]          val;
      int            num, i;
      long           pos, from, to;

      retval = null;
      try {
        if (keyword != null && this.m_index != null && this.m_arraybegin > 0) {
          val = (int[]) this.m_index[keyword.ToUpper()];
          if (val != null) {
            pos = this.m_arraybegin + val[0] * 6;
            num = val[1];
            if (num > 0) {
              if (this.m_indexfilereader == null) {
                this.m_indexfilereader =
                        new BinaryReader(File.OpenRead(this.m_filename));
              }
              this.m_indexfilereader.BaseStream.Position = pos;
              if (this.m_dictfilestream == null) {
                this.m_dictfilestream = File.OpenRead(this.m_dictfilename);
              }
              retval = new string[num];
              for(i = 0; i < num; i++) {
                from = (long) this.m_indexfilereader.ReadInt32();
                to = from + (long) this.m_indexfilereader.ReadInt16() + 1;
                der = new DictEntryRef(from, to);
                retval[i] = der.read(this.m_dictfilestream);
              }
            } // if (num > 0)
          } // if (val != null)
        }
      }
      catch(Exception) {}
      return retval;
    } // get()

    /// <summary>
    /// Returns translations for all keywords meeting the given regular
    /// expression
    /// </summary>
    /// <remarks>
    /// Searches keywords meeting the given regular expression and finds
    /// entries for them using get() for each keyword found.
    /// </remarks>
    /// <param name="keywordre">
    /// the regular expression for kewords to translate
    /// </param>
    /// <returns>
    /// Array of results for keywords meeting reg. expr.
    /// The method may also return null, if nothing has been found for
    /// the given regular expression.
    /// </returns>
    public string[] getre(string keywordre) {
      Regex     re;
      ArrayList al;
      string[]  retval, res;
      int       i, j;

      al = new ArrayList();
      if (this.m_index != null && keywordre != null) {
        re = new Regex(keywordre.ToUpper());
        foreach(object key in this.m_index.Keys) {
          if (re.IsMatch(key.ToString())) al.Add(key);
        }
      }
      retval = new string[DictImport.MAXENTRIES];
      i = 0;
      foreach(object key in al) {
        if (i >= DictImport.MAXENTRIES) break;
        res = this.get(key.ToString());
        if (res != null) {
          for(j = 0; j < res.Length && i < DictImport.MAXENTRIES; i++, j++)
            retval[i] = res[j];
        }
      } // foreach
      if (i < DictImport.MAXENTRIES && i > 0) {
        res = retval;
        retval = new string[i];
        Array.Copy(res, retval, i);
        res = null;
      }
      return retval;
    }

    /// <summary>Reads an object from the given file</summary>
    /// <remarks>
    /// The opposite of DictImport.saveIdx. Reads the index located
    /// in the first part of the index file into the internal map
    /// m_index and estimates the position of the begin of the DictEntryRef
    /// array in m_arraybegin. The progress of the reading process is
    /// populated using IProgress.progress() of the progress handler
    /// m_progresshandler. After the method is ready, it calls
    /// IProgress.ready(), either with null as argument, if everything went
    /// OK or with the exception string, if errors occured.
    /// The path for the dictionary file located in the index file may be
    /// evtl. icorrect, if the dictionary file can't be found, then the
    /// method tries to find it in the dicrectory of the index file or in the
    /// directory of the executing assembly.
    /// </remarks>
    /// <seealso cref="DictImport.saveIdx">DictImport.saveIdx</seealso>
    private void readObj() {
      BinaryReader   r;
      Exception      e;
      string         key, fn, dir;
      byte[]         barr;
      int[]          val;
      int            num, strlen, i, perc, percold;

      r = null; e = null;
      try {
        barr = new byte[256];
        r = new BinaryReader(File.OpenRead(this.m_filename));
        num = r.ReadInt32();
        this.m_index = new Hashtable(num);
        strlen = r.ReadInt32();
        if (strlen > barr.Length) barr = new byte[strlen]; 
        strlen = r.Read(barr, 0, strlen);
        this.m_dictfilename = DictEntryRef.DICT_ENCODING.GetString(barr, 0, strlen);
        // correct dictionary file name, if the dictionary file doesn't exist
        if (!File.Exists(this.m_dictfilename)) {
          fn = Path.GetFileName(this.m_dictfilename);
          dir = Path.GetDirectoryName(this.m_filename);
          this.m_dictfilename = Path.Combine(dir, fn);
          if (!File.Exists(this.m_dictfilename)) {
            dir = Utils.Exedir;
            this.m_dictfilename = Path.Combine(dir, fn);
          }
        }
        percold = 0;
        for(i = 1; i <= num; i++) {
          strlen = r.ReadInt32();
          if (strlen > barr.Length) barr = new byte[strlen]; 
          strlen = r.Read(barr, 0, strlen);
          key = DictEntryRef.DICT_ENCODING.GetString(barr, 0, strlen);
          val = new int[2];
          val[0] = r.ReadInt32(); // the begin index
          val[1] = r.ReadInt32(); // number of DictEntryRef entries
          this.m_index.Add(key, val);
          perc = (int) (i * 100 / num);
          if (perc != percold) {
            percold = perc;
            if (this.m_progresshandler != null) this.m_progresshandler.progress(perc);
          }
        } // for i
        this.m_arraybegin = r.BaseStream.Position;
      }
      catch(Exception excp) {
        e = excp;
      }
      if (r != null) {
        try { r.Close(); } catch(Exception) {}
      }
      if (this.m_progresshandler != null) {
        if (e == null) {
          if (this.m_goon) this.m_progresshandler.ready(null);
        }
        else {
          if (this.m_goon) this.m_progresshandler.ready(e.ToString());
        }
      } // if (this.m_progresshandler != null)
    } // readObj()

    /// <summary>Releases the index object</summary>
    /// <remarks>Joins evtl. threads, closes files, releases index map</remarks>
    public void Dispose() {
      this.stop();
      if (this.m_indexfilereader != null) {
        try { this.m_indexfilereader.Close(); } catch(Exception) {}
        this.m_indexfilereader = null;
      }
      if (this.m_dictfilestream != null) {
        try { this.m_dictfilestream.Close(); } catch(Exception) {}
        this.m_dictfilestream = null;
      }
      if (this.m_index != null) {
        this.m_index.Clear();
        this.m_index = null;
      }
    } // Dispose()

  } // class DictIndex
} // namespace dict

