using System.IO;
using System.Text;

namespace dict {

  /// <summary>Dict.cc dictionary file parser</summary>
  /// <remarks>
  /// Extracts keywords and locations of the explanation for these
  /// keywords from a dictionary file obtained from dict.cc.
  /// The following format of the dictionary file is expected:
  /// <list type="bullet">
  /// <item>
  ///  The dictionary file is a text file encoded in DictEntryRef.DICT_ENCODING
  ///  (usually cp1252).
  /// </item>
  /// <item>
  ///  One translation is located in one line of this text file.
  /// </item>
  /// <item>
  ///  Each line consists of two parts separated with INDEX_SEPARATOR (::).
  ///  The left part contains text in the source language, for example
  ///  English in the English-German dictionary, the right part contains
  ///  the translation in the target language.
  /// </item>
  /// <item>
  ///  The class splits the left part into keywords. It uses SEPARATORS
  ///  (white spaces, tabulators, semicolons, slashes, dots, commas etc.)
  ///  in order to extract single keywords from the whole sentence.
  ///  Text passages in the left part located between OPENING and CLOSING
  ///  (usually between brackets, or HTML sequences like between an ampersound
  ///  and a semicolon) are ignored.
  /// </item>
  /// </list>
  /// For each extracted keyword IDictParseHandler.addIndex() is called,
  /// with the extracted keyword and the position from and to of the whole
  /// line in the dictionary file (in bytes). The location from and to can
  /// be used for example in the class DictEntryRef in order to read the
  /// whole line from the dictionary file and return it as string.
  /// </remarks>
  /// <seealso cref="DictImport">DictImport</seealso>
  /// <seealso cref="DictEntryRef">DictEntryRef</seealso>
  public class DictParser {

    /// <summary>Length of the buffer used for reading input file</summary>
    private const int BUFLEN = 2048;

    /// <summary>Ignore lines starting with these characters</summary>
    private const string STARTLINE_IGNORE = @"#.";

    /// <summary>
    /// Characters beginning strings to ignore while parsing for index
    /// </summary>
    private const string OPENING = @"<({[&";

    /// <summary>
    /// Characters closing strings to ignore while parsing for index
    /// </summary>
    private const string CLOSING = @">)}];";

    /// <summary>
    /// Strings separating index for the remaining part of the entry
    /// </summary>
    private const string INDEX_SEPARATOR = @"::";

    /// <summary>
    /// Characters separating indexes from each other
    /// </summary>
    private const string SEPARATORS = " \t.,;:/";

    /// <summary>Handler for retrieving extracted indexes</summary>
    private IDictParserHandler m_handler;

    /// <summary>Initializes the parser for the given handler</summary>
    /// <param name="handler">
    /// the handler for retrieving extracted indexes
    /// </param>
    public DictParser(IDictParserHandler handler) {
      this.m_handler = handler;
    } // DictParser()

    /// <summary>Extracts indexes from one single dictionary line</summary>
    /// <param name="s">the dictionary line string</param>
    /// <param name="from">position of the line begin in the file</param>
    /// <param name="to">position of the line end in the file</param>
    /// <returns>
    /// true, if the process should continue, false if it was interrupted by
    /// by the user
    /// </returns>
    private bool processLine(string s, long from, long to) {
      StringBuilder sb, sbres;
      string        index;
      char          c;
      int           i, j, idxsep, len;
      bool          found, retval;

      retval = true;
      if (s != null) {
        idxsep = s.IndexOf(DictParser.INDEX_SEPARATOR);
        if (idxsep != -1) {
          sb = new StringBuilder(s.Substring(0, idxsep).Trim());
          len = sb.Length;
          if (len > 0) {
            c = sb[0];
            if (DictParser.STARTLINE_IGNORE.IndexOf(c) == -1) {
              do {
                found = false;
                for(i = 0; i < len; i++) {
                  c = sb[i];
                  if (DictParser.OPENING.IndexOf(c) != -1) {
                    for(j = i + 1; j < len; j++) {
                      c = sb[j];
                      if (DictParser.CLOSING.IndexOf(c) != -1) {
                        found = true;
                        sb.Remove(i, j - i + 1);
                        len = sb.Length;
                        break;
                      }
                    } // for j
                    break;
                  } // if
                } // for i
              } while(found);
              sbres = new StringBuilder();
              len = sb.Length;
              for(i = 0; i < len; i++) {
                c = sb[i];
                if (DictParser.SEPARATORS.IndexOf(c) != -1) {
                  if (sbres.Length > 0) {
                    if (this.m_handler != null) {
                      index = sbres.ToString().ToUpper();
                      retval = this.m_handler.addIndex(index, from, to);
                    }
                    sbres.Length = 0;
                  }
                }
                else
                  sbres.Append(c);
              } // for i
              if (sbres.Length > 0) {
                if (this.m_handler != null) {
                  index = sbres.ToString().ToUpper();
                  retval = this.m_handler.addIndex(index, from, to);
                }
              }
            } // if
          } // if (len > 0)
        } // if (idxsep != -1)
      } // if (s != null)
      return retval;
    } // processLine()

    /// <summary>Extracts indexes from the given stream</summary>
    /// <remarks>
    /// Format of the dictionary file as in dict.cc.
    /// </remarks>
    /// <param name="input">the stream to read the dictionary file from</param>
    public void read(Stream input) {
      byte[] bin, bstr;
      string s;
      int    actidx, i, len;
      long   offset, offsetold;
      byte   b;

      if (input != null) {
        bin = new byte[DictParser.BUFLEN];
        bstr = new byte[DictParser.BUFLEN];
        actidx = 0;
        offset = 0;
        offsetold = 0;
        while ((len = input.Read(bin, 0, bin.Length)) >0) {
          for(i = 0; i < len; i++, offset++) {
            b = bin[i];
            if (b == 10 || b == 13) {
              if (actidx > 0) {
                s = DictEntryRef.DICT_ENCODING.GetString(bstr, 0, actidx);
                actidx = 0;
                this.processLine(s, offsetold, offset);
              }
              offsetold = offset+1;
            }
            else {
              if (actidx < DictParser.BUFLEN) {
                bstr[actidx] = b;
                actidx++;
              }
            }
          } // for i
        } // while
        if (actidx > 0) {
          s = DictEntryRef.DICT_ENCODING.GetString(bstr, 0, actidx);
          this.processLine(s, offsetold, offset);
        }
      } // if (input != null)
    } // read()
  } // class DictParser

} // namespace dict

